import React from 'react';
import loadModule from '../utils/loadModule';
import getLanguages from '../utils/getLanguages';

/**
 * A React effect hook that highlights code in a given element.

 * - Specify `{isContainer: true}` if your element is not the target, but instead contains target elements. (Uses `Prism.highlightAllUnder` instead of `Prism.highlightElement`)
 * - Specify `{useWorker: true}` to set the `async` flag in the prism highlighter. Only useful for large blocks of code.
 * - Check out the [prism documentation](https://prismjs.com/extending.html#api) for more details
 *
 * @param {Object} ref a React ref to the target element
 * @param {Object} options the options object
 * @param {String} [options.language = 'javascript'] name of the prism language to load
 * @param {String} [options.theme = 'okaidia'] name of the prism theme to load
 * @param {Boolean} [options.isContainer] whether the given element ref is a container
 * @param {Boolean} [options.useWorker] whether to use a web worker for highlighting
 * @param {Function} [options.callback] invoked when prism is done highlighting
 *
 */
export default function usePrism(ref, options) {
    React.useEffect(() => {
        try {
            applyPrism(ref, options);
        } catch (error) {
            console.warn('[usePrism] failed', { error });
        }
    }, [
        ref.current || ref,
        options.language,
        options.theme,
        options.isContainer,
        options.useWorker,
        options.callback
    ]);
}

async function applyPrism(
    ref,
    {
        language = 'javascript',
        theme = 'okaidia',
        isContainer = false,
        useWorker = false,
        callback = undefined
    } = {}
) {
    const isMounted = () => !!(ref && ref.current);
    const Prism = await import(/* webpackChunkName: 'codeblock/prismjs' */ 'prismjs');

    if (isMounted() && theme) {
        await loadModule('theme', theme);
    }

    if (isMounted() && language) {
        await loadModule('language', language);
    }

    if (isMounted() && isContainer) {
        for (const lang of getLanguages(ref.current)) {
            if (isMounted() && lang) {
                await loadModule('language', lang);
            }
        }
    }

    if (isMounted()) {
        isContainer
            ? Prism.highlightAllUnder(ref.current, useWorker, callback)
            : Prism.highlightElement(ref.current, useWorker, callback);
    }
}
